/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.kio.admin.file.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef INTERFACE_FILE_H
#define INTERFACE_FILE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.kde.kio.admin.File
 */
class OrgKdeKioAdminFileInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.kde.kio.admin.File"; }

public:
    OrgKdeKioAdminFileInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgKdeKioAdminFileInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> close()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("close"), argumentList);
    }

    inline QDBusPendingReply<> open()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("open"), argumentList);
    }

    inline QDBusPendingReply<> read(qulonglong size)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(size);
        return asyncCallWithArgumentList(QStringLiteral("read"), argumentList);
    }

    inline QDBusPendingReply<> seek(qulonglong offset)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(offset);
        return asyncCallWithArgumentList(QStringLiteral("seek"), argumentList);
    }

    inline QDBusPendingReply<qulonglong> size()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("size"), argumentList);
    }

    inline QDBusPendingReply<> truncate(qulonglong length)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(length);
        return asyncCallWithArgumentList(QStringLiteral("truncate"), argumentList);
    }

    inline QDBusPendingReply<> write(const QByteArray &data)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(data);
        return asyncCallWithArgumentList(QStringLiteral("write"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void closed();
    void data(const QByteArray &data);
    void mimeTypeFound(const QString &mimeType);
    void opened();
    void positionChanged(qulonglong offset);
    void result(int error, const QString &errorMessage);
    void truncated(qulonglong length);
    void written(qulonglong written);
};

#endif
