/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.kio.admin.statcommand.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef INTERFACE_STATCOMMAND_H
#define INTERFACE_STATCOMMAND_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.kde.kio.admin.StatCommand
 */
class OrgKdeKioAdminStatCommandInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.kde.kio.admin.StatCommand"; }

public:
    OrgKdeKioAdminStatCommandInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgKdeKioAdminStatCommandInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> start()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("start"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void result(int error, const QString &errorString);
};

#endif
