/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.kio.admin.mkdircommand.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#include "interface_mkdircommand.h"

/*
 * Implementation of interface class OrgKdeKioAdminMkdirCommandInterface
 */

OrgKdeKioAdminMkdirCommandInterface::OrgKdeKioAdminMkdirCommandInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent)
    : QDBusAbstractInterface(service, path, staticInterfaceName(), connection, parent)
{
}

OrgKdeKioAdminMkdirCommandInterface::~OrgKdeKioAdminMkdirCommandInterface()
{
}


#include "moc_interface_mkdircommand.cpp"
