/****************************************************************************
** Meta object code from reading C++ file 'file.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kio-admin-26.04.0/src/helper/file.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'file.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN4FileE_t {};
} // unnamed namespace

template <> constexpr inline auto File::qt_create_metaobjectdata<qt_meta_tag_ZN4FileE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "File",
        "D-Bus Interface",
        "org.kde.kio.admin.File",
        "opened",
        "",
        "data",
        "mimeTypeFound",
        "mimeType",
        "written",
        "closed",
        "positionChanged",
        "offset",
        "truncated",
        "length",
        "result",
        "error",
        "errorMessage",
        "open",
        "read",
        "size",
        "write",
        "close",
        "seek",
        "truncate"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'opened'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'data'
        QtMocHelpers::SignalData<void(const QByteArray &)>(5, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 5 },
        }}),
        // Signal 'mimeTypeFound'
        QtMocHelpers::SignalData<void(const QString &)>(6, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'written'
        QtMocHelpers::SignalData<void(qulonglong)>(8, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 8 },
        }}),
        // Signal 'closed'
        QtMocHelpers::SignalData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'positionChanged'
        QtMocHelpers::SignalData<void(qulonglong)>(10, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 11 },
        }}),
        // Signal 'truncated'
        QtMocHelpers::SignalData<void(qulonglong)>(12, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 13 },
        }}),
        // Signal 'result'
        QtMocHelpers::SignalData<void(int, const QString &)>(14, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 }, { QMetaType::QString, 16 },
        }}),
        // Slot 'open'
        QtMocHelpers::SlotData<void()>(17, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'read'
        QtMocHelpers::SlotData<void(qulonglong)>(18, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 19 },
        }}),
        // Slot 'write'
        QtMocHelpers::SlotData<void(const QByteArray &)>(20, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 5 },
        }}),
        // Slot 'close'
        QtMocHelpers::SlotData<void()>(21, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'seek'
        QtMocHelpers::SlotData<void(qulonglong)>(22, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 11 },
        }}),
        // Slot 'truncate'
        QtMocHelpers::SlotData<void(qulonglong)>(23, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 13 },
        }}),
        // Slot 'size'
        QtMocHelpers::SlotData<qulonglong()>(19, 4, QMC::AccessPublic, QMetaType::ULongLong),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<File, qt_meta_tag_ZN4FileE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject File::staticMetaObject = { {
    QMetaObject::SuperData::link<BusObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4FileE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4FileE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4FileE_t>.metaTypes,
    nullptr
} };

void File::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<File *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->opened(); break;
        case 1: _t->data((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 2: _t->mimeTypeFound((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->written((*reinterpret_cast<std::add_pointer_t<qulonglong>>(_a[1]))); break;
        case 4: _t->closed(); break;
        case 5: _t->positionChanged((*reinterpret_cast<std::add_pointer_t<qulonglong>>(_a[1]))); break;
        case 6: _t->truncated((*reinterpret_cast<std::add_pointer_t<qulonglong>>(_a[1]))); break;
        case 7: _t->result((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 8: _t->open(); break;
        case 9: _t->read((*reinterpret_cast<std::add_pointer_t<qulonglong>>(_a[1]))); break;
        case 10: _t->write((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 11: _t->close(); break;
        case 12: _t->seek((*reinterpret_cast<std::add_pointer_t<qulonglong>>(_a[1]))); break;
        case 13: _t->truncate((*reinterpret_cast<std::add_pointer_t<qulonglong>>(_a[1]))); break;
        case 14: { qulonglong _r = _t->size();
            if (_a[0]) *reinterpret_cast<qulonglong*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (File::*)()>(_a, &File::opened, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (File::*)(const QByteArray & )>(_a, &File::data, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (File::*)(const QString & )>(_a, &File::mimeTypeFound, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (File::*)(qulonglong )>(_a, &File::written, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (File::*)()>(_a, &File::closed, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (File::*)(qulonglong )>(_a, &File::positionChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (File::*)(qulonglong )>(_a, &File::truncated, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (File::*)(int , const QString & )>(_a, &File::result, 7))
            return;
    }
}

const QMetaObject *File::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *File::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4FileE_t>.strings))
        return static_cast<void*>(this);
    return BusObject::qt_metacast(_clname);
}

int File::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = BusObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void File::opened()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void File::data(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void File::mimeTypeFound(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void File::written(qulonglong _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void File::closed()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void File::positionChanged(qulonglong _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void File::truncated(qulonglong _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void File::result(int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}
QT_WARNING_POP
